#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>

#define BUF_SIZE        64

#define GPIO_PIN        10
#define GPIO_DIR        "/sys/class/gpio"
#define GPIO_IN         0
#define GPIO_OUT        1

static int
gpio_export (unsigned int gpio)
{
  int fd, len;
  char buf[BUF_SIZE];

  fd = open (GPIO_DIR "/export", O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/export");
      return fd;
    }

  len = snprintf (buf, sizeof(buf), "%d", gpio);
  write (fd, buf, len);
  close (fd);

  return 0;
}

static int
gpio_unexport (unsigned int gpio)
{
  int fd, len;
  char buf[BUF_SIZE];

  fd = open (GPIO_DIR "/unexport", O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/unexport");
      return fd;
    }

  len = snprintf (buf, sizeof(buf), "%d", gpio);
  write (fd, buf, len);
  close (fd);

  return 0;
}

static int
gpio_set_direction (unsigned int gpio,
                    unsigned int direction)
{
  int fd;
  char buf[BUF_SIZE];

  snprintf (buf, sizeof(buf), GPIO_DIR "/gpio%d/direction", gpio);

  fd = open (buf, O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/direction");
      return fd;
    }

  if (direction)
    write (fd, "out", sizeof("out"));
  else
    write (fd, "in", sizeof("in"));

  close (fd);
  return 0;
}

static int
gpio_set_value (unsigned int gpio,
                unsigned int value)
{
  int fd;
  char buf[BUF_SIZE];

  snprintf (buf, sizeof(buf), GPIO_DIR "/gpio%d/value", gpio);

  fd = open (buf, O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/set-value");
      return fd;
    }

  if (value)
    write (fd, "1", 2);
  else
    write (fd, "0", 2);

  close (fd);
  return 0;
}

int
main (void)
{
  if (gpio_export (GPIO_PIN) < 0)
    exit (EXIT_FAILURE);

  if (gpio_set_direction (GPIO_PIN, GPIO_OUT) < 0)
    exit (EXIT_FAILURE);

  /* infinite loop */
  while (1)
    {
      gpio_set_value (GPIO_PIN, 1);
      sleep (1);

      gpio_set_value (GPIO_PIN, 0);
      sleep (1);
    }

  return EXIT_SUCCESS;
}
