#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <poll.h>

#define BUF_SIZE        64

#define GPIO_PIN        3
#define GPIO_DIR        "/sys/class/gpio"
#define GPIO_IN         0
#define GPIO_OUT        1

static int
gpio_export (unsigned int gpio)
{
  int fd, len;
  char buf[BUF_SIZE];

  fd = open (GPIO_DIR "/export", O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/export");
      return fd;
    }

  len = snprintf (buf, sizeof(buf), "%d", gpio);
  write (fd, buf, len);
  close (fd);

  return 0;
}

static int
gpio_unexport (unsigned int gpio)
{
  int fd, len;
  char buf[BUF_SIZE];

  fd = open (GPIO_DIR "/unexport", O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/unexport");
      return fd;
    }

  len = snprintf (buf, sizeof(buf), "%d", gpio);
  write (fd, buf, len);
  close (fd);

  return 0;
}

static int
gpio_set_direction (unsigned int gpio,
                    unsigned int direction)
{
  int fd;
  char buf[BUF_SIZE];

  snprintf (buf, sizeof(buf), GPIO_DIR "/gpio%d/direction", gpio);

  fd = open (buf, O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/direction");
      return fd;
    }

  if (direction)
    write (fd, "out", sizeof("out"));
  else
    write (fd, "in", sizeof("in"));

  close (fd);
  return 0;
}

static int
gpio_set_edge (unsigned int  gpio,
               char         *edge)
{
  int fd;
  char buf[BUF_SIZE];

  snprintf (buf, sizeof(buf), GPIO_DIR "/gpio%d/edge", gpio);

  fd = open (buf, O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/edge");
      return fd;
    }

  write (fd, edge, strlen(edge) + 1);
  close (fd);

  return 0;
}

static int
gpio_set_value (unsigned int gpio,
                unsigned int value)
{
  int fd;
  char buf[BUF_SIZE];

  snprintf (buf, sizeof(buf), GPIO_DIR "/gpio%d/value", gpio);

  fd = open (buf, O_WRONLY);
  if (fd < 0)
    {
      perror ("gpio/set-value");
      return fd;
    }

  if (value)
    write (fd, "1", 2);
  else
    write (fd, "0", 2);

  close (fd);
  return 0;
}

static int
gpio_fd_open (unsigned int gpio)
{
  int fd;
  char buf[BUF_SIZE];

  snprintf (buf, sizeof(buf), GPIO_DIR "/gpio%d/value", gpio);

  fd = open (buf, O_RDONLY | O_NONBLOCK );
  if (fd < 0)
    perror ("gpio/fd_open");

  return fd;
}

int
main (void)
{
  struct pollfd fdset[1];
  int nfds = 1;
  int fd, ret;
  char buf[BUF_SIZE];

  if (gpio_export (GPIO_PIN) < 0)
    exit (EXIT_FAILURE);

  if (gpio_set_direction (GPIO_PIN, GPIO_IN) < 0)
    exit (EXIT_FAILURE);

  if (gpio_set_edge (GPIO_PIN, "falling") < 0)
    exit (EXIT_FAILURE);

  fd = gpio_fd_open (GPIO_PIN);
  if (fd < 0)
    exit (EXIT_FAILURE);

  lseek (fd, 0, SEEK_SET);
  read (fd, &buf, BUF_SIZE);

  while (1)
    {
      memset (fdset, 0, sizeof(fdset));

      fdset[0].fd = fd;
      fdset[0].events = POLLPRI;

      ret = poll (fdset, nfds, -1);
      if (ret < 0)
        {
          printf ("poll(): failed!\n");
          goto exit;
        }

      if (fdset[0].revents & POLLPRI)
        {
          printf ("poll(): GPIO_%d interrupt occurred\n", GPIO_PIN);
          lseek (fdset[0].fd, 0, SEEK_SET);
          read (fdset[0].fd, &buf, BUF_SIZE);
        }

       fflush(stdout);
    }

exit:
  close (fd);
  return EXIT_FAILURE;
}
